/* Linear programing for special case of equality constriants m=m3 */
/* From numerical recipes */

#include <math.h> 
#include "qnrutil.h"

#define EPS1 1.0e-9
#define EPS2 1.0e-9
#define FREEALL free_ivector(l2,1,m); free_ivector(l1,1,n+1);

int simplx(double **a,int m,int n,int *izrov,int *iposv)
{
        int i,ip,ir,is,k,kh,kp,m12,nl1,nl2;
        int *l1,*l2;
        double q1,bmax;
        void simp1(),simp2(),simp3();

        l1=ivector(1,n+1);
        l2=ivector(1,m);
        nl1=n;
        for (k=1;k<=n;k++) l1[k]=izrov[k]=k;
        nl2=m;
        for (i=1;i<=m;i++) {
                if (a[i+1][1] < 0.0) nrerror("Bad input tableau in SIMPLX");
                l2[i]=i;
                iposv[i]=n+i;
        }
/* stage one */
        ir=1;
        for (k=1;k<=(n+1);k++) {
                q1=0.0;
                for (i=1;i<=m;i++) q1 += a[i+1][k];
                a[m+2][k] = -q1;
        }
        do {
                simp1(a,m+1,l1,nl1,0,&kp,&bmax);
                if (bmax <= EPS1 && a[m+2][1] < -EPS1) {
			FREEALL
			return -1;
                } else if (bmax <= EPS1 && a[m+2][1] <= EPS1) {
                        for (ip=1;ip<=m;ip++) {
                                if (iposv[ip] == (ip+n)) {
                                        simp1(a,ip,l1,nl1,1,&kp,&bmax);
                                        if (bmax > 0.0) goto one;
                                }
                        }
                        ir=0;
                        break;
                }
                simp2(a,n,l2,nl2,&ip,kp,&q1);
                if (ip == 0) {
			FREEALL
                        return -1;
                }
one:    simp3(a,m+1,n,ip,kp);
                if (iposv[ip] >= (n+1)) {
                        for (k=1;k<=nl1;k++)
                                if (l1[k] == kp) break;
                        --nl1;
                        for (is=k;is<=nl1;is++) l1[is]=l1[is+1];
                        a[m+2][kp+1] += 1.0;
                        for (i=1;i<=m+2;i++) a[i][kp+1] = -a[i][kp+1];
                }
                is=izrov[kp];
                izrov[kp]=iposv[ip];
                iposv[ip]=is;
        } while (ir);
/* phase two */
	for (;;) {
		simp1(a,0,l1,nl1,0,&kp,&bmax);
		if (bmax <= 0.0) {
			FREEALL
                        return 0;
		}
		simp2(a,n,l2,nl2,&ip,kp,&q1);
		if (ip == 0) {
			FREEALL
                        return 1;
		}
		simp3(a,m,n,ip,kp);
		is=izrov[kp];
		izrov[kp]=iposv[ip];
		iposv[ip]=is;
	}
}

void simp1(double **a,int mm,int *ll,int nll,int iabf,int *kp,double *bmax)
{
        int k;
        double test;

        *kp=ll[1];
        *bmax=a[mm+1][*kp+1];
        for (k=2;k<=nll;k++) {
                if (iabf == 0)
                        test=a[mm+1][ll[k]+1]-(*bmax);
                else
                        test=fabs(a[mm+1][ll[k]+1])-fabs(*bmax);
                if (test > 0.0) {
                        *bmax=a[mm+1][ll[k]+1];
                        *kp=ll[k];
                }
        }
}

void simp2(double **a,int n,int *l2,int nl2,int *ip,int kp,double *q1)
{
        int k,ii,i;
        double qp,q0,q;

        *ip=0;
        for (i=1;i<=nl2;i++) {
                if (a[l2[i]+1][kp+1] < -EPS2) {
                        *q1 = -a[l2[i]+1][1]/a[l2[i]+1][kp+1];
                        *ip=l2[i];
                        for (i=i+1;i<=nl2;i++) {
                                ii=l2[i];
                                if (a[ii+1][kp+1] < -EPS2) {
                                        q = -a[ii+1][1]/a[ii+1][kp+1];
                                        if (q < *q1) {
                                                *ip=ii;
                                                *q1=q;
                                        } else if (q == *q1) {
                                          for (k=1;k<=n;k++) {
                                            qp = -a[*ip+1][k+1]/a[*ip+1][kp+1];
                                            q0 = -a[ii+1][k+1]/a[ii+1][kp+1];
                                            if (q0 != qp) break;
                                          }
                                          if (q0 < qp) *ip=ii;
                                        }
                                }
                        }
                }
        }
}

void simp3(double **a,int i1,int k1,int ip,int kp)
{
        int kk,ii;
        double piv;

        piv=1.0/a[ip+1][kp+1];
        for (ii=1;ii<=i1+1;ii++)
                if (ii-1 != ip) {
                        a[ii][kp+1] *= piv;
                        for (kk=1;kk<=k1+1;kk++)
                                if (kk-1 != kp)
                                        a[ii][kk] -= a[ip+1][kk]*a[ii][kp+1];
                }
        for (kk=1;kk<=k1+1;kk++)
                if (kk-1 != kp) a[ip+1][kk] *= -piv;
        a[ip+1][kp+1]=piv;
}

#undef EPS1
#undef EPS2
#undef FREEALL
