/* the simpicial method */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "constype.h"
#include "filenames.h"

#include "init.h"
#include "qnrutil.h"
#include "collect.h"
#include "result.h"
#include "points.h"


static int run_boundary()
{
 char command_line[LINE_LENGTH];

 strcpy(command_line, param.spice_name);
 strcat(command_line, MARGINS_CALL);

 return system(command_line);
}


static int compute_boundary(fp)
FILE *fp;
{
 SIGNALS *sp;
 char inputfile[NAME_LENGTH];

 strcpy(inputfile, param.tmp_dir);
 strcat(inputfile, param.circuit_name);
 strcat(inputfile, param.nom_ext);


 switch(param.pulse_extraction_method)
  {
   case COLLECT_BY_PHASE:
        sp=collect_pulses_by_phase(inputfile);
        break;

   case COLLECT_BY_VOLTAGE:
        sp=collect_pulses_by_voltage(inputfile);
        break;
  }

 if(sp!=NULL)
  {
   if(sp->clk_num>0)
    {
     change_into_logic(sp);
     print_with_clk(sp, fp);
    }
   else
     print_async_signals(sp, fp);
  }
 else
  return 0;

 return 1;
}



static int store_boundary_logic(param_nr, highmarg, pseudomarg)
int param_nr, highmarg, pseudomarg;
{
 FILE *fp;
 double paramvalue;
 int i, x;

 char logicfile[LINE_LENGTH];
 char errmsg[LINE_LENGTH];

 strcpy(errmsg, "Can't write to the '");
 strcat(errmsg, PARAM_FILE);
 strcat(errmsg, "' file.\n");

 for(i=0; i<2; i++)
  {
   if((fp=fopen(PARAM_FILE,"w")) == NULL)
       nrerror(errmsg);

   for(x=1; x<=dim; x++)
    {
     paramvalue = hullpnts[pntcount][x]*scale[x];
     if(!pseudomarg)
       if(!i)
         paramvalue -= delta[x];
       else
         paramvalue += delta[x];
     fprintf(fp, "param[%d] = %.6lf\n", x, paramvalue);
    }
   fprintf(fp, "set circuit = '%s'\n", param.circuit_name);
   fprintf(fp, "set opt_num = '%s'\n", param.opt_num);

   fclose(fp);

   run_boundary();

   strcpy(logicfile, param.output_dir);
   strcat(logicfile, param.circuit_name);
   strcat(logicfile, param.bound_ext);
   strcat(logicfile, param.logic_ext);
   strcat(logicfile, param.opt_num);

   if((fp=fopen(logicfile, "a"))==NULL)
    {
     printf("Cannot open file %s!\n\n", logicfile);
     return 0;
    }

   fprintf(fp, "PARAMETER %d  -  %s:\n", param_nr, names[param_nr-1]);
   if(!highmarg)
     fprintf(fp, "LOW MARGIN - ");
   else
     fprintf(fp, "HIGH MARGIN - ");
   if(!i || pseudomarg)
     fprintf(fp, "POINT WITHIN OPERATING REGION\n\n");
   else
     fprintf(fp, "OUTSIDE OPERATING REGION\n\n");
   for(x=1; x<=dim; x++)
    {
     paramvalue = hullpnts[pntcount][x]*scale[x];
     if(!pseudomarg)
       if(!i)
         paramvalue -= delta[x];
       else
         paramvalue += delta[x];
     fprintf(fp, "param[%d] = %.6lf", x, paramvalue);
     if(x==param_nr)
       {
	fprintf(fp, "\t<= %6.2lf%s", 
                    100*(paramvalue/scale[x] - centerpnt[x])/centerpnt[x],
                    "%");
        if(pseudomarg)
	  fprintf(fp, "*");
       }
     fprintf(fp, "\n");
    }
   fprintf(fp, "\n\n");
   
   compute_boundary(fp);
   fprintf(fp, "\n\n");
   fclose(fp);
   if(pseudomarg)
	break;
 }
}




static int margcomp(pm1, pm2)
MARGIN *pm1, *pm2;
{
 if(pm1->val > pm2->val)
  return 1;
 else
 if(pm1->val < pm2->val)
  return -1;
 else
  return 0;
}


static int read_command_line(argc, argv)
int argc;
char *argv[];
{
 if((argc<3) || (argc>4))
   return 0;

 if(argc==3)
  if(argv[1][0]!='-')
   return 1;
  else
   return 0;

 if((argc==4) && (argv[2][0]!='-'))
  {
   if(strcmp(argv[1], "-b")==0)
    {
     param.input_options |= B_SET;
     return 1;
    }
   else 
   if(strcmp(argv[1], "-2")==0)
    {
     param.input_options |= TWO_SET;
     return 1;
    }
   else
   if(strcmp(argv[1], "-B")==0)
    {
     param.input_options |= BATCH_SET;
     return 1;
    }
   else 
    return 0;
  }
 else
  return 0;
}

#ifdef MAIN
#define marg_main main
#endif

int marg_main(argc, argv)
int argc;
char *argv[];
{
/* load filename variables and stuff (Kris') */

	init_parameters();

 	if(!read_command_line(argc, argv))
  	  {
   	   printf("\nSYNOPSIS:\n\n");
           printf("\tmarg [-B|-b|-2] <circuit_name> <parameter_set_number>\n\n");
   	   exit(255);
    	  }

	if(!read_config(INIT_CONFIG_FILE))
		exit(255);

/* read name of the circuit */
        strncpy(param.circuit_name, argv[argc-2], NAME_LENGTH);

/* read name of input number */
	strncpy(param.opt_num, argv[argc-1], NAME_LENGTH);

        if(param.input_options & BATCH_SET)
          strcat(param.spice_name, " -b ");


	generate_jspice_config();

	param.program = MARG_PROGRAM;

/* read input file, create and initialize global arrays */
	readinit();

	if(param.input_options & TWO_SET)
          get_regions();
	else
          compute_margins();
    return (0);
}


int compute_margins()
{
 	FILE *fp;
	char outfile[LINE_LENGTH];
	char errmsg[LINE_LENGTH];
        char outline[LONG_LINE_LENGTH];
	int x, y;

	MARGIN *margins;
	int margcount=0;
	double *pc, *po;



#define SCREEN_FILE_WRITE                               \
                        printf("%s", outline);          \
			fprintf(fp, "%s", outline);


	pc=vector(1,dim);
	po=vector(0,dim);
        hullpnts  = matrix(1,dim*2,0,dim);
        margins = (MARGIN*)calloc(2*dim, sizeof(MARGIN));


   	if(param.input_options & B_SET) {
		/* reset file with the description of circuit behavior
			at parameter region boundaries */
		strcpy(outfile, param.output_dir);
		strcat(outfile, param.circuit_name);
		strcat(outfile, param.bound_ext);
		strcat(outfile, param.logic_ext);
	        strcat(outfile, param.opt_num);
	        if ((fp=fopen(outfile,"w")) == NULL) {
	 	 	strcpy(errmsg, "Can't write to the '");
			strcat(errmsg, outfile);
	 		strcat(errmsg, "' file.\n");
                	nrerror(errmsg);
	        }
		fclose(fp);
	}

	strcpy(outfile, param.output_dir);
	strcat(outfile, param.circuit_name);
	strcat(outfile, param.marg_ext);
	strcat(outfile, param.opt_num);

        if ((fp=fopen(outfile,"w")) == NULL) {
                strcpy(errmsg, "Can't write to the '");
		strcat(errmsg, outfile);
		strcat(errmsg, "' file.\n");
                nrerror(errmsg);
        }


/* find initial dim*2 points, print margins */
/* turn the margins into points, print margins */
	sprintf(outline, "CIRCUIT NAME:\t%s\n\n", param.circuit_name);
        SCREEN_FILE_WRITE
        printf("Margins calculated to within +/-%5.2f percent.\n",param.accuracy);
        fprintf(fp,"Margins calculated to within +/-%5.2f percent.\n",param.accuracy);
        printf("Param:      Margins:                               Percent:\n");
        fprintf(fp,"Param:      Margins:                               Percent:\n");

        for (x=1; dim >= x; ++x) {
                for (y=1; dim >= y; ++y)
                        pc[y]=po[y]=centerpnt[y];

                pc[x]=centerpnt[x];
                po[x]=lower[x];

                if (!addpoint(pc,po))
                        nrerror("parameter values failed");

                else {
			sprintf(outline, "%s\t%9.5f ...%9.5f ...",
				names[x-1],
				hullpnts[pntcount][x]*scale[x],
				centerpnt[x]*scale[x]);
                        SCREEN_FILE_WRITE

			margins[margcount].val=
		  100.0*(centerpnt[x]-hullpnts[pntcount][x])/centerpnt[x];
			margins[margcount].sgn=-1;
			margins[margcount].param_nr=x-1;
			margins[margcount++].pseudo=
				(hullpnts[pntcount][x]==lower[x])?1:0;

   			if(param.input_options & B_SET)
				store_boundary_logic(x, 0, margins[margcount-1].pseudo);
		}

                pc[x]=centerpnt[x];
                po[x]=upper[x];

                if (!addpoint(pc,po))
                        nrerror("parameter values failed");

                else {
			sprintf(outline, 
                                "%9.5f      ", hullpnts[pntcount][x]*scale[x]);
                        SCREEN_FILE_WRITE

			margins[margcount].val=
		100.0*(hullpnts[pntcount][x]-centerpnt[x])/centerpnt[x];
			margins[margcount].sgn=+1;
			margins[margcount].param_nr=x-1;
			margins[margcount++].pseudo=
				(hullpnts[pntcount][x]==upper[x])?1:0;

			sprintf(outline, "-%5.1f%s  +%5.1f%s\t",
				margins[margcount-2].val,
				margins[margcount-2].pseudo?"*":" ",
				margins[margcount-1].val,
				margins[margcount-1].pseudo?"*":" "
				);
			SCREEN_FILE_WRITE

			sprintf(outline, "\n");
			SCREEN_FILE_WRITE

   			if(param.input_options & B_SET)
		 		store_boundary_logic(x, 1, margins[margcount-1].pseudo);
		}
        }

 qsort(margins, margcount, sizeof(MARGIN), (int (*)())margcomp);
 printf("\n\nMost critical margins:\n\n");
 fprintf(fp,"\n\nMost critical margins:\n\n");
 for(x=0; x<margcount; x++)
  {
   sprintf(outline, "%d.   %s\t=>\t%s%5.1f%s\n",
           x+1, names[margins[x].param_nr], 
	   ((margins[x].sgn>0)?"+":"-"), margins[x].val,
	   (margins[x].pseudo?"*":" "));
   SCREEN_FILE_WRITE

   if(x==4) {
     printf("\n");
     fprintf(fp,"\n");
   }
  }
}
