/* the simpicial method */

/* minimum distance out from old plane */
#define STEP (param.accuracy/200.0)
/* distance out from search boundary */
#define OUT  (param.accuracy/200.0)
#define DEFINED 1.0e-18
#define INSCRIBE 1.0e-8

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "qnrutil.h"
#include "qdet.h"
#include "dsimplx.h"
#include "constype.h"
#include "filenames.h"
#include "init.h"
#include "points.h"
#include "yield.h"

void intpickpnts(int, int *);
void makeaplane(int *);
int findface(double *,double *);
int postcenter(int *,double *);
int postshift(int *,double *);
double det(double **, int);
int simplx(double **,int,int,int *,int *);
void make_more_mem(void);

static int **abpnts;
static double **ab;
static char filename[LINE_LENGTH], NAMES[80];
static int plncount=0, maxplane=10;
static int stop=0;
static FILE *fpout;
static char outline[LONG_LINE_LENGTH];
static double **aNmatrix, *temp;

#define OUT_SCREEN_FILE                                 \
                        printf("%s", outline);          \
			fprintf(fpout, "%s", outline);

static int read_opt_command_line(int argc, char *argv[])
{
 if((argc<3) || (argc>4))
   return 0;

 if(argc==3)
  if(argv[1][0]!='-')
   return 1;
  else
   return 0;

 if((argc==4) && (argv[2][0]!='-'))
  {
   if(strcmp(argv[1], "-B")==0)
    {
     param.input_options |= BATCH_SET;
     return 1;
    }
   else 
    return 0;
  }
 else
  return 0;
}

#ifdef MAIN
#define opt_main main
#endif

int opt_main(int argc, char *argv[])
{
	FILE *fp, *ftest;
	char outfile[LINE_LENGTH];
	double *pc, *po, *facecenter, *radius, *direction;
	double bound, cbig, c, out, distance, step, dist;
	int *pntstack, *tang;
	int x, y, xx;
	int good, count, big, oldcount, lowcount, concave;
	int iterate=0, direcount=0;
	double yest1=1.0, yest3=1.0, *ypnt;
	double gpdf3, gpdf1, total3, total1, ycount3, ycount1, gint3, gint1;
	int outside;
	long tick, z, seed=-1;

/* load filename variables and stuff (Kris's) */
        init_parameters();

/* check syntax */
 	if(!read_opt_command_line(argc, argv)) {
           printf("SYNOPSIS:\n\topt [-B] <circuit_name> <parameter_set_number>\n\n");
           exit(255);
        }

        if(!read_config(INIT_CONFIG_FILE)) {
		printf("Can't read the '");
		printf(INIT_CONFIG_FILE);
		printf("' file.\n");
                exit(255);
	}
/* read name of input file */
        strncpy(param.circuit_name, argv[argc-2], NAME_LENGTH);
/* read name of input number */
        strncpy(param.opt_num, argv[argc-1], NAME_LENGTH);

        if(param.input_options & BATCH_SET)
           strcat(param.spice_name, " -b ");

        generate_jspice_config();

	param.program = OPT_PROGRAM;

/* create the ITERATE file*/
        if ((fp=fopen("ITERATE","w")) == NULL) {
                printf("Can't write the 'ITERATE' file.\n");
		exit(255);
	}
        fprintf(fp,"Delete this file to terminate the optimization program intelligently.\n");
        fclose(fp);
/* read input file, create and initialize global arrays */
	readinit();

	for (x=1; dim >= x; ++x)  maxplane *=2;
        ab=matrix(1,maxplane,0,dim);
        abpnts=imatrix(1,maxplane,0,dim);
	hullpnts=matrix(1,param.maxiter+3*dim+1,1,dim);
	aNmatrix=matrix(1,dim,1,dim);
	temp=vector(0,dim);
/* open the output file */
	strcpy(outfile, param.output_dir);
        strcat(outfile, param.circuit_name);
        strcat(outfile, param.opt_ext);
        strcat(outfile, param.opt_num);

        if ((fpout=fopen(outfile,"w")) == NULL) {
                printf("Can't write to the '");
                printf(outfile);
                printf("' file.\n");
                exit(255);
        }
/* check values of MIN_ITER and MAX_ITER */
        if (param.miniter > param.maxiter) {
                sprintf(outline,"MIN_ITER = %d is greater than MAX_ITER = %d.  MAX_ITER will supercede.\n",param.miniter,param.maxiter);
                OUT_SCREEN_FILE
        }

/* initialize local arrays */
        facecenter=vector(1,dim);
        radius=vector(0,dim+1);
        for (x=0; dim+1 >= x; ++x)
                radius[x]=0.0;
        direction=vector(1,dim);
        for (x=1; dim >= x; ++x)
                direction[x]=0.0;
        pntstack=ivector(1,dim);
	pc=vector(1,dim);
	po=vector(0,dim);
/* find initial dim*2 points, print margins */
/* turn the margins into points, print margins */
	sprintf(outline, "Margins calculated to within +/-%5.2f percent.\n",param.accuracy);
        OUT_SCREEN_FILE
	sprintf(outline, "        Margins:                                 Percent:\n");
        OUT_SCREEN_FILE
        for (x=1; dim >= x; ++x) {
                for (y=1; dim >= y; ++y)
                        pc[y]=po[y]=centerpnt[y];
                pc[x]=centerpnt[x]-STEP/scale[x];
                po[x]=lower[x]-OUT/scale[x];
                if (!addpoint(pc,po))
                        nrerror("parameter values failed");
                else {
                        sprintf(outline, "%s\t%9.5f%s...%9.5f ...",
                                names[x-1],
                                hullpnts[pntcount][x]*scale[x],
                                (hullpnts[pntcount][x]<lower[x])?"*":" ",
                                centerpnt[x]*scale[x]);
                        OUT_SCREEN_FILE
                }
                pc[x]=centerpnt[x]+STEP/scale[x];
                po[x]=upper[x]+OUT/scale[x];
                if (!addpoint(pc,po))
                        nrerror("parameter values failed");
                else {
                        sprintf(outline, "%9.5f%s     -%5.1f  +%5.1f\n",
                                hullpnts[pntcount][x]*scale[x],
                                (hullpnts[pntcount][x]>upper[x])?"*":" ",
                                100.0*(centerpnt[x]-hullpnts[pntcount-1][x])/centerpnt[x],
                                100.0*(hullpnts[pntcount][x]-centerpnt[x])/centerpnt[x]
                                );
                        OUT_SCREEN_FILE
                }
        }
/* pick combinations of dim points and make the hull */
	intpickpnts(1,pntstack);
	sprintf(outline, "hull planes: %i total\n\n",plncount);
	OUT_SCREEN_FILE

/* LOOP */
	do {
/* stick a sphere inside the scaled hull (same as an ellipse) */
		sprintf(outline, "%i\n", ++iterate);
                OUT_SCREEN_FILE

		if (!(big=findface(facecenter, radius))) {
			sprintf(outline, "Could not expand in any critical direction.\n");
			OUT_SCREEN_FILE
			stop=2;
		}

		if (!stop) {
/* print the results */
		sprintf(outline, "center    ");
		OUT_SCREEN_FILE
		for (x=1; dim >= x; ++x) {
			sprintf(outline, "%9.4f ",centerpnt[x]*scale[x]);
			OUT_SCREEN_FILE
		}
		sprintf(outline, "\naxes      ");
		OUT_SCREEN_FILE
		for (x=1; dim >= x; ++x) {
			sprintf(outline, "%9.3f ",radius[0]*scale[x]);
			OUT_SCREEN_FILE
		}
		sprintf(outline, "\ndirection ");
		OUT_SCREEN_FILE
		for (x=1; dim >= x; ++x) {
			sprintf(outline, "%9.3f ",c=-ab[big][x]);
			OUT_SCREEN_FILE
			direction[x] +=c*c;
		}
		sprintf(outline, "\n");
		OUT_SCREEN_FILE
		++direcount;
/* find the closest boundary and calculate the search points */
		cbig=0.0;
		for (x=1; dim >= x; ++x) {
			bound=((ab[big][x] > 0.0) ? lower[x] : upper[x]);
			if ((c=ab[big][x]/(facecenter[x]-bound)) > cbig)
				cbig=c;
		}
/* pc on plane, po on the boundary */
/* in 1-D step=STEP/scale[x] */
/* in 2-D step equals smallest of the above */
		step = STEP/scale[x];
		for (x=2; dim >= x; ++x)
			if (STEP/scale[x] < step)
				step = STEP/scale[x];
/* check that we are not against search boundary */
                for (x=1; dim >= x; ++x) {
                        out=facecenter[x]-2*step*ab[big][x];
                        if ((out>=upper[x])||(out<=lower[x]))
                                abpnts[big][0]=1;
                }
                if (abpnts[big][0]) {
                        sprintf(outline, "Could not expand in above direction.\n");
                        OUT_SCREEN_FILE
                } else {
 
/* pc and po shifted out a little bit */
                for (x=1; dim >= x; ++x) {
                	pc[x]=facecenter[x]-step*ab[big][x];
                	po[x]=facecenter[x]-ab[big][x]/cbig-OUT/scale[x]*ab[big][x];
		}
		lowcount=oldcount=plncount;
/* flag plane if not convex */
		if (!addpoint(pc,po)) {
			abpnts[big][0]=1;
			sprintf(outline, "New point is not outside hull.\n");
			OUT_SCREEN_FILE
		} else {
/* print the new point */
		sprintf(outline, "new point ");
		OUT_SCREEN_FILE
		for (x=1; dim >= x; ++x) {
			sprintf(outline,"%9.3f%s",
			hullpnts[pntcount][x]*scale[x],
			(hullpnts[pntcount][x]<lower[x])||(hullpnts[pntcount][x]>upper[x])?"*":" "
				);
			OUT_SCREEN_FILE
		}
		sprintf(outline, "\n");
		OUT_SCREEN_FILE
/* flag planes the new point is exterior to */
		for (y=1; oldcount >= y; ++y) {
			distance=ab[y][0];
			for (x=1; dim >= x; ++x)
				distance +=ab[y][x]*hullpnts[pntcount][x];
			if (distance < 0.0) {
				abpnts[y][0]=2;
				--lowcount;
			}
		}
/* find new planes, using new point and old plane combinations */
        	for (xx=1; oldcount >= xx; ++xx) {
                	if (abpnts[xx][0] == 2) {
                        	for (x=1; x <= dim; ++x) {
                                	for (y=1; y <= dim; ++y)
                                        	pntstack[y] = abpnts[xx][y];
                                	pntstack[x] = pntcount;
                                	makeaplane(pntstack);
/* memory */
					if (plncount == maxplane)
						make_more_mem();
                        	}
                	}
        	}
/* delete old planes */
		for (y=1; oldcount >= y; ) {
			if (abpnts[y][0] == 2) {
				for (x=0; dim >= x; ++x) {
					ab[y][x]=ab[plncount][x];
					abpnts[y][x]=abpnts[plncount][x];
				}
				--plncount;
			} else ++y;
		}

		}
		}
		}
		sprintf(outline, "hull planes: %i deleted, %i added, %i total\n",oldcount-lowcount, plncount-lowcount,plncount);
		OUT_SCREEN_FILE
/* check if loop should terminate */
/* maximum number of iterations */
		if (iterate == param.maxiter) {
			stop=2;
                        sprintf(outline, "Number of iterations equals MAX_ITER.\n");
                        OUT_SCREEN_FILE
                }
/* maximum number of planes */
                if (stop == 3) {
      			sprintf(outline,"Number of hull planes exceeds MAX_PLANE.\n");
                        OUT_SCREEN_FILE
		}
/* radius */
		if (!(stop>=2)&&(radius[0] - radius[dim+1])/radius[0] < STEP*2) {
			stop=1;
			sprintf(outline, "Axes of inscribed ellipse increasing slowly.\n");
			OUT_SCREEN_FILE
		}
		for (x=dim+1; 1 <= x; --x)
			radius[x]=radius[x-1];
/* minimum number of iterations */
		if (!(stop>=2) && param.miniter > iterate)  stop=0;
/* ITERATE file gone */
                if ((fp=fopen("ITERATE","r")) == NULL) {
                        stop=2;
                        sprintf(outline, "The ITERATE file was deleted.\n");
			OUT_SCREEN_FILE
		}
                fclose(fp);
	} while (!stop);
/* end loop */
	sprintf(outline, "\nResults.");
	OUT_SCREEN_FILE
/* inscribe one more time */
	tang=ivector(1,dim+1-pin);
	postcenter(tang, radius);
        sprintf(outline, "\nCENTER    ");
        OUT_SCREEN_FILE
        for (x=1; dim >= x; ++x) {
                sprintf(outline, "%9.4f ",centerpnt[x]*scale[x]);
                OUT_SCREEN_FILE
	}
        sprintf(outline, "\nAXES      ");
        OUT_SCREEN_FILE
        for (x=1; dim >= x; ++x) {
                sprintf(outline, "%9.3f ",radius[0]*scale[x]);
                OUT_SCREEN_FILE
	}
	sprintf(outline, "\nAVE DIRECTION ");
	OUT_SCREEN_FILE
        for (x=1; dim >= x; ++x) {
                sprintf(outline, "%5.3f     ",sqrt(direction[x]/direcount));
                OUT_SCREEN_FILE
	}

        sprintf(outline, "\n\ncriticalness ");
        OUT_SCREEN_FILE
        for (x=1; dim >= x; ++x) {
                direction[x]=0.0;
                for (y=1; dim+1-pin >= y; ++y)
                        direction[x] +=(ab[tang[y]][x])*(ab[tang[y]][x]);
                sprintf(outline, "%6.3f    ",sqrt(direction[x]/(dim+1-pin)));
                OUT_SCREEN_FILE
        }

/* convexity check along critical vectors */
	sprintf(outline, "\n\ndistance to boundary (normalized) ... critical direction\n");
	OUT_SCREEN_FILE
	for (y=1; dim+1-pin >= y; ++y) {
		for (x=1; dim >= x; ++x) {
/* find the closest boundary and calculate the search points */
               		cbig=0.0;
                	for (xx=1; dim >= xx; ++xx) {
                        	bound=((ab[tang[y]][xx] > 0.0) ? lower[xx] : upper[xx]);
                        	if ((c=ab[tang[y]][xx]/(centerpnt[xx]-bound)) > cbig)
                                	cbig=c;
                	}
/* pc at center, po on boundary */
                        pc[x]=centerpnt[x];
                        po[x]=centerpnt[x]-ab[tang[y]][x]/cbig-OUT/scale[x]*ab[tang[y]][x];
		}
/* find boundary */
               	if (!addpoint(pc,po))
			nrerror("parameter values failed");
		else {
			dist=0.0;
                        for (x=1; dim >= x; ++x)
				dist +=(hullpnts[pntcount][x]-centerpnt[x])*(hullpnts[pntcount][x]-centerpnt[x]);
                        dist=sqrt(dist);
                        sprintf(outline, "%6.4f  ... ",(dist/radius[0]));
                        OUT_SCREEN_FILE
			for (x=1; dim >= x; ++x) {
                		sprintf(outline, "%7.3f   ",-ab[tang[y]][x]);
                		OUT_SCREEN_FILE
			}
		}
		sprintf(outline, "\n");
        	OUT_SCREEN_FILE
	}
	sprintf(outline, "\n");
        OUT_SCREEN_FILE
/* yield calculations assuming three and one sigma */
	if (var > 0) {
		sprintf(outline, "Yield calculation for parameter(s):\t");
                OUT_SCREEN_FILE
                for (x=1; dim >= x; ++x)  if (yield[x]) {
			sprintf(outline, "%s\t",names[x-1]);
                	OUT_SCREEN_FILE
		}

		sprintf(outline, "\nAssuming 1 sigma variations of    :\t");
                OUT_SCREEN_FILE
                for (x=1; dim >= x; ++x)  if (yield[x]) {
                        sprintf(outline, "%4.1f\t",1.0/(3.0*centerpnt[x]));
                        OUT_SCREEN_FILE
                }
		
/* ellipse estimate */
		yest3=multiarea(300.0*radius[0],var);
		yest1=multiarea(100.0*radius[0],var);
/* hull estimate */
		tick=0;
		ypnt=vector(1,dim);
                total3=total1=ycount3=ycount1=0.0;
                for (z=1; 10000 >= z && tick <= 1000 && (z-tick) <= 1000; ++z) {
                        outside=0;
			gpdf3=gpdf1=1.0;
                        for (x=1; dim >= x; ++x) {
				ypnt[x]=(yield[x])?lower[x]+(upper[x]-lower[x])*uniform_deviate(&seed):centerpnt[x];
				gpdf3 *=normal(300.0*(ypnt[x]-centerpnt[x]));
				gpdf1 *=normal(100.0*(ypnt[x]-centerpnt[x]));
			}
                        for (y=1; plncount >= y && !outside; ++y) {
                                distance=ab[y][0];
                                for (x=1; dim >= x; ++x)
                                        distance +=ab[y][x]*ypnt[x];
                                if (distance < 0.0)  outside=1;
                        }
                        if (!outside) {
				ycount3 +=gpdf3;
				ycount1 +=gpdf1;
				++tick;
			}
			total3 +=gpdf3;
			total1 +=gpdf1;
                }
		gint3 = gint1 = 1.0; 
                for (x=1; dim >= x; ++x) {
			gint3 *=area(300.0*(upper[x]-centerpnt[x]),300.0*(centerpnt[x]-lower[x]));
			gint1 *=area(100.0*(upper[x]-centerpnt[x]),100.0*(centerpnt[x]-lower[x]));
		}
		sprintf(outline, "\nYield within ellipse, yield within hull:  %7.3f  %5.1f",100.0*yest3,100.0*gint3*(ycount3/total3));
                OUT_SCREEN_FILE
		sprintf(outline, "\n\nAssuming 1 sigma variations of    :\t");
                OUT_SCREEN_FILE
		for (x=1; dim >= x; ++x)  if (yield[x]) {
                        sprintf(outline, "%4.1f\t",1.0/centerpnt[x]);
                        OUT_SCREEN_FILE
                }

		sprintf(outline, "\nYield within ellipse, yield within hull:  %7.3f  %5.1f\n\n",100.0*yest1,100.0*gint1*(ycount1/total1));
                OUT_SCREEN_FILE
	}
	fclose(fpout);
    return (0);
}

int postcenter(int *tang, double *radius)
{
        double **tab;
        int *right, *left;
        int x, y;
 
/* inscribe the sphere */
/* initialize the input tabeau */
        tab=matrix(1,dim+3,1,plncount+1+2*pin);
        left=ivector(1,dim+1);
        right=ivector(1,plncount+2*pin);
 
        for (y=0; dim >= y; ++y)
                tab[y+1][1]=0.0;
        tab[dim+2][1]=1.0;
        for (x=1; plncount >= x; ++x)
                tab[1][x+1]=-ab[x][0];
        for (x=1; pin >= x; ++x) {
                tab[1][x*2+plncount]=-centerpnt[xyz[x]]-INSCRIBE;
                tab[1][x*2+1+plncount]=centerpnt[xyz[x]]-INSCRIBE;
        }
 
        for (x=1; plncount >= x; ++x)
                tab[dim+2][x+1]=-1.0;
        for (x=plncount+1; plncount+2*pin >= x; ++x)
                tab[dim+2][x+1]=0.0;
 
        for (y=1; dim >= y; ++y)
                for (x=1; plncount >= x; ++x)
                        tab[y+1][x+1]=ab[x][y];
        for (y=1; dim >= y; ++y)
                for (x=1; pin >= x; ++x) {
                        tab[y+1][x*2+plncount]=(y == xyz[x]?-1.0:0.0);
                        tab[y+1][x*2+1+plncount]=(y == xyz[x]?1.0:0.0);
                }
/* call the linear program */
        if (0 != simplx(tab,dim+1,plncount+2*pin,right,left))
                nrerror("inscribing in hull failed");
        radius[0]=-tab[1][1];
        for (y=1; dim >= y; ++y)
                for (x=1; plncount+2*pin >= x; ++x)
                        if (right[x] == y+plncount+2*pin)
                                centerpnt[y]=-tab[1][x+1];
	x=0;
        for (y=1; dim+1 >= y; ++y)
		if (left[y] <= plncount)
                	tang[++x]=left[y];

        free_matrix(tab,1,dim+3,1,plncount+1+2*pin);
        free_ivector(left,1,dim+1);
        free_ivector(right,1,plncount+2*pin);
}

static int iter=0;
int findface(double *centface,double *radius)
{
	double **tab, *sine;
	double cos, facevalue=0.0;
	int *right, *left, *tang, *intsect;
	int x, y, z, fail, plane, yy, share, noshare, count, face=0, tnum;

/* inscribe the sphere */
/* initialize the input tabeau */
	tab=matrix(1,dim+3,1,plncount+1+2*pin);
	left=ivector(1,dim+1);
	right=ivector(1,plncount+2*pin);
	sine=vector(1,dim+1);
	tang=ivector(1,dim+1-pin);
	intsect=ivector(1,dim+1);

	for (y=0; dim >= y; ++y)
		tab[y+1][1]=0.0;
	tab[dim+2][1]=1.0;
	for (x=1; plncount >= x; ++x)
		tab[1][x+1]=-ab[x][0];
	for (x=1; pin >= x; ++x) {
		tab[1][x*2+plncount]=-startpnt[xyz[x]]-INSCRIBE;
		tab[1][x*2+1+plncount]=startpnt[xyz[x]]-INSCRIBE;
	}

	for (x=1; plncount >= x; ++x)
		tab[dim+2][x+1]=-1.0;
	for (x=plncount+1; plncount+2*pin >= x; ++x)
		tab[dim+2][x+1]=0.0;

	for (y=1; dim >= y; ++y)
		for (x=1; plncount >= x; ++x)
			tab[y+1][x+1]=ab[x][y];
	for (y=1; dim >= y; ++y)
		for (x=1; pin >= x; ++x) {
			tab[y+1][x*2+plncount]=(y == xyz[x]?-1.0:0.0);
			tab[y+1][x*2+1+plncount]=(y == xyz[x]?1.0:0.0);
		}
/* call the linear program */
	if (0 != simplx(tab,dim+1,plncount+2*pin,right,left)) {
		stop=2;
		sprintf(outline, "Hull simplex failed.\n");
		OUT_SCREEN_FILE
		goto bailed;
	}
	radius[0]=-tab[1][1];
	for (y=1; dim >= y; ++y)
        	for (x=1; plncount+2*pin >= x; ++x)
			if (right[x] == y+plncount+2*pin)
				centerpnt[y]=-tab[1][x+1];
        x=0;
        for (y=1; dim+1 >= y; ++y)
                if (left[y] <= plncount)
			if (x+1 > dim+1-pin) {
				sprintf(outline,"One tangent plane is being ignored\n");
				OUT_SCREEN_FILE
                        } else  tang[++x]=left[y];

	if ((tnum=x) != dim+1-pin) {
		  sprintf(outline,"One or more tangent planes are missing.\n");
                  OUT_SCREEN_FILE
        }
/* find largest of the tangetial faces */
	for (z=1; tnum >= z; ++z) {
		plane=tang[z];
		if (!abpnts[plane][0]) {

/* find the intersecting planes, which share N-1 points (noshare is 1) */
		count=0;
		for (x=1; plncount >= x; ++x) {
			noshare=0;
			for (y=1; noshare < 2 && dim >= y; ++y) {
				share = 0;
				for (yy=1; !share && dim >= yy; ++yy)
					share=(abpnts[plane][yy]==abpnts[x][y]);
				if (!share)  ++noshare;
			}
			if (noshare == 1)
				if (++count <= dim)
					intsect[count]=x;
		}
		if (count != dim) {
			abpnts[plane][0]=1;
		} else {

		intsect[dim+1]=plane;
/* find the angles for the plane */
		sine[dim+1]=0.0;
		for (y=1; dim >= y; ++y) {
			cos=0.0;
			for (x=1; dim >= x; ++x)
				cos=cos+ab[intsect[y]][x]*ab[plane][x];
			sine[y]=sqrt(1.0 - cos*cos);
		}
        	for (y=0; dim >= y; ++y)
                	tab[y+1][1]=0.0;
        	tab[dim+2][1]=1.0;
        	for (x=1; dim+1 >= x; ++x)
                	tab[1][x+1]=-ab[intsect[x]][0];
		for (x=1; dim+1 >= x; ++x)
			tab[dim+2][x+1]=-sine[x];
        	for (y=1; dim >= y; ++y)
                	for (x=1; dim+1 >= x; ++x)
                        	tab[y+1][x+1]=ab[intsect[x]][y];
        	for (y=0; dim+1 >= y; ++y)
			tab[y+1][dim+3]=-tab[y+1][dim+2];
		tab[1][dim+2] -=INSCRIBE;
		tab[1][dim+3] -=INSCRIBE;
/* call the linear program */
		if (0 != simplx(tab,dim+1,dim+2,right,left)) {
			abpnts[plane][0]=1;
		} else {
/* save face is the biggest so far */
			if (-tab[1][1] > facevalue) {
				facevalue=(-tab[1][1]);
				face=plane;
				for (y=1; dim >= y; ++y)
        				for (x=1; dim+2 >= x; ++x)
						if (right[x] == y+dim+2)
        						centface[y]=-tab[1][x+1];
			}
		}

		}
		if (abpnts[plane][0]==1) {
			sprintf(outline, "          ");
                        OUT_SCREEN_FILE
			for (x=1; dim >= x; ++x) {
                        	sprintf(outline, "%9.3f ",-ab[plane][x]);
                        	OUT_SCREEN_FILE
			}
			sprintf(outline, "\nCould not expand in above direction.\n");
                        OUT_SCREEN_FILE
		}

		}
	}
	bailed:
	free_matrix(tab,1,dim+3,1,plncount+1+2*pin);
	free_ivector(left,1,dim+1);
	free_ivector(right,1,plncount+2*pin);
	free_vector(sine,1,dim+1);
	free_ivector(tang,1,dim+1-pin);
	free_ivector(intsect,1,dim+1);
	return face;
}

void intpickpnts(int depth, int *pp)
{
        for (pp[depth]=depth*2-1; depth*2 >= pp[depth]; ++pp[depth])
		if (depth == dim)
			makeaplane(pp);
		else
			intpickpnts(depth+1, pp);
}

void makeaplane(int *pp)
{
	double distance, sumsqr, norm;
	int n, a, y, posd=0, negd=0, pntinpln, def;

/* B */
	for (a=1; dim >= a; ++a)
		for (y=1; dim >= y; ++y)
			aNmatrix[a][y]=hullpnts[pp[a]][y];
	temp[0]=det(aNmatrix,dim);
/* A */
	for (n=1; dim >= n; ++n) {
		for (a=1; dim >= a; ++a) {
			for (y=1; dim >= y; ++y)
				aNmatrix[a][y]=hullpnts[pp[a]][y];
			aNmatrix[a][n]=-1.0;
		}
		temp[n]=det(aNmatrix,dim);
	}
/* check that plane is well defined */
	for (n=1; !(def=(fabs(temp[n]) > DEFINED)) && dim >= n; ++n);

	if (def) {

/* check that all points (not in the plane) are on the same side of the plane */
	for (y=1; !(posd && negd) && pntcount >= y; y++) {
		for (a=1; !(pntinpln=(y == pp[a])) && dim >= a; a++);
		if (!pntinpln) {
			distance=temp[0];
			for (n=1; dim >= n; ++n) 
				distance +=temp[n]*hullpnts[y][n];
			if (distance >= 0.0) posd=1;
			else  negd=1;
		}
	}
/* save the good planes */
	if (!(posd && negd)) {
		++plncount;
/* normalize */
		sumsqr=0.0;
		for (n=1; n <= dim; n++)
			sumsqr +=temp[n]*temp[n];
		norm=1/(sqrt(sumsqr));
		for (n=0; n <= dim; n++)
			temp[n] *=norm;
		distance=temp[0];
		for (n=1; n <= dim; n++)
			distance +=temp[n]*centerpnt[n];
		if (distance < 0.0)
			for (n=0; n <= dim; n++)
				temp[n] *=-1.0;
/* store plane in global array */
		for (n=0; n <= dim; n++)
			ab[plncount][n]=temp[n];
		abpnts[plncount][0]=0;
		for (n=1; n <= dim; n++)
			abpnts[plncount][n]=pp[n];
	}
	}
}

void make_more_mem(void)
{
	int newmax;

	if (maxplane >= param.maxplane) {
		stop=3;
		newmax = maxplane+1000;
	} else {
		newmax=maxplane*2;
		if (newmax > param.maxplane)  newmax=param.maxplane;
	}
	ab=reallocmatrix(ab,1,maxplane,newmax,0,dim);
	abpnts=ireallocmatrix(abpnts,1,maxplane,newmax,0,dim);
	maxplane = newmax;
}
