/* Array and matrix assignment routines from numerical recipes */
#include <stdio.h>
#include <stdlib.h>

#include "lineread.h"

	
int *ivector(int nl,int nh)
{
	int *v;
	v=(int *)malloc((unsigned) (nh-nl+1)*sizeof(int));
	if (!v) nrerror("allocation failure in ivector()");
	return v-nl;
}

double *vector(int nl,int nh)
{
        double *v;
        v=(double *)malloc((unsigned) (nh-nl+1)*sizeof(double));
        if (!v) nrerror("allocation failure in vector()");
        return v-nl;
}

int **imatrix(int nrl,int nrh, int ncl, int nch)
{
        int i, **m;
 
        m=(int **) malloc((unsigned) (nrh-nrl+1)*sizeof(int*));
        if (!m) nrerror("allocation failure 1 in imatrix()");
        m -= nrl;
 
        for(i=nrl;i<=nrh;i++) {
                m[i]=(int *) malloc((unsigned) (nch-ncl+1)*sizeof(int));
                if (!m[i]) nrerror("allocation failure 2 in imatrix()");
        m[i] -= ncl;
        }
        return m;
}
 
double **matrix(int nrl,int nrh, int ncl, int nch)
{
	int i;
	double **m;

	m=(double **) malloc((unsigned) (nrh-nrl+1)*sizeof(double*));
	if (!m) nrerror("allocation failure 1 in matrix()");
	m -= nrl;

	for(i=nrl;i<=nrh;i++) {
		m[i]=(double *) malloc((unsigned) (nch-ncl+1)*sizeof(double));
		if (!m[i]) nrerror("allocation failure 2 in matrix()");
	m[i] -= ncl;
	}
	return m;
}

int **ireallocmatrix(int **oldm,int nrl,int nrh,int newnrh,int ncl,int nch)
{
        int i, **m;
 
        m=(int **) realloc((char*) (oldm+nrl), (unsigned) (newnrh-nrl+1)*sizeof(int*));
        if (!m) nrerror("allocation failure 1 in imatrix()");
        m -= nrl;
 
        for(i=nrh+1;i<=newnrh;i++) {
                m[i]=(int *) malloc((unsigned) (nch-ncl+1)*sizeof(int));
                if (!m[i]) nrerror("allocation failure 2 in imatrix()");
        m[i] -= ncl;
        }
        return m;
}
 
double **reallocmatrix(double **oldm, int nrl,int nrh, int newnrh, int ncl, int nch)
{
        int i;
	double **m;
 
        m=(double **) realloc((char*) (oldm+nrl), (unsigned) (newnrh-nrl+1)*sizeof(double*));
        if (!m) nrerror("allocation failure 1 in matrix()");
        m -= nrl;
 
        for(i=nrh+1;i<=newnrh;i++) {
                m[i]=(double *) malloc((unsigned) (nch-ncl+1)*sizeof(double));
                if (!m[i]) nrerror("allocation failure 2 in matrix()");
        m[i] -= ncl;
        }
        return m;
}

void free_ivector(int *v,int nl,int nh)
{
	free((char*) (v+nl));
}

void free_vector(double *v,int nl,int nh)
{
        free((char*) (v+nl));
}

void free_imatrix(int **m,int nrl,int nrh,int ncl,int nch)
{
        int i;

        for(i=nrh;i>=nrl;i--) free((char *) (m[i]+ncl));
        free((char*) (m+nrl));
}
 
void free_matrix(double **m,int nrl,int nrh,int ncl,int nch)
{
        int i;
 
        for(i=nrh;i>=nrl;i--) free((char *) (m[i]+ncl));
        free((char*) (m+nrl));
}
