/* the simpicial method */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "constype.h"
#include "qnrutil.h"
#include "points.h"

#define DEBUG     0
#define DEBUG_ADD 0


struct REGION_CRV_STRUCT
{
 double dist;
 double alpha;
 double delta;
 int start_pnt;
 int end_pnt;
 struct REGION_CRV_STRUCT *next;
};

typedef struct 
{
 double x;
 double y;
 double mx;
 double my;
 double alpha;
} REGION_PNT;

typedef struct REGION_CRV_STRUCT REGION_CRV;


static REGION_CRV *largest_crv;
static double minimum_alpha;


static void add_to_crv_list(REGION_CRV *new)
{
 REGION_CRV *tmp;


 if(largest_crv==NULL) {
  largest_crv=new;
  return;
 }

 if(new->dist>largest_crv->dist) {
  new->next=largest_crv;
  largest_crv = new;
  return;
 }

 tmp = largest_crv;
 while(tmp->next!=NULL) {
  if(new->dist>tmp->next->dist) {
    new->next=tmp->next;
    tmp->next=new;
    break;
  }
  tmp=tmp->next;
 }
   
 if(tmp->next==NULL) 
   tmp->next=new;
}

#if DEBUG_PREVIOUS

static int modify_region_crv_list(int x, int y)
{
 REGION_CRV *nc1, *nc2, *tmp;
 double dist;

 nc1 = (REGION_CRV *)calloc(1, sizeof(REGION_CRV));
 nc2 = (REGION_CRV *)calloc(1, sizeof(REGION_CRV));

 if((nc1==NULL) || (nc2==NULL))
   return 0;

 nc1->alpha = largest_crv->alpha - largest_crv->delta;
 nc1->start_pnt = largest_crv->start_pnt;
 nc1->end_pnt = pntcount;
 nc1->delta = largest_crv->delta/2;
 nc1->dist = 0; 
 nc1->dist += (hullpnts[pntcount][x] - hullpnts[nc1->start_pnt][x])*
	      (hullpnts[pntcount][x] - hullpnts[nc1->start_pnt][x]);
 nc1->dist += (hullpnts[pntcount][y] - hullpnts[nc1->start_pnt][y])*
	      (hullpnts[pntcount][y] - hullpnts[nc1->start_pnt][y]);

 nc2->start_pnt = pntcount;
 nc2->end_pnt   = largest_crv->end_pnt;
 nc2->alpha     = largest_crv->alpha + largest_crv->delta;
 nc2->delta     = largest_crv->delta/2;
 nc2->dist = 0; 
 nc2->dist += (hullpnts[nc2->end_pnt][x] - hullpnts[pntcount][x])*
	      (hullpnts[nc2->end_pnt][x] - hullpnts[pntcount][x]);
 nc2->dist += (hullpnts[nc2->end_pnt][y] - hullpnts[pntcount][y])*
	      (hullpnts[nc2->end_pnt][y] - hullpnts[pntcount][y]);

 tmp = largest_crv;
 largest_crv = largest_crv->next;
 free(tmp);

 add_to_crv_list(nc1);
 add_to_crv_list(nc2);
 return 1;
}

#endif


static double angle(double x, double y)
/* translation of the tangent of the angle into angle in the range of
   0 .. 2*PI */
{
 double alpha;

 alpha = atan(y/x);
 if(x<0)
   alpha+=M_PI;
 if(alpha<0)
   alpha+=2*M_PI;

 return alpha;
}


static int modify_region_crv_list(int x, int y)
{
 REGION_CRV *nc1, *nc2, *tmp;
 double dist;
 double new_point_x, new_point_y;
 double last_alpha;

 nc1 = (REGION_CRV *)calloc(1, sizeof(REGION_CRV));
 nc2 = (REGION_CRV *)calloc(1, sizeof(REGION_CRV));

 if((nc1==NULL) || (nc2==NULL))
   return 0;

 nc1->start_pnt = largest_crv->start_pnt;
 nc1->end_pnt = pntcount;

 new_point_x = (hullpnts[nc1->end_pnt][x] + hullpnts[nc1->start_pnt][x])/2;
 new_point_y = (hullpnts[nc1->end_pnt][y] + hullpnts[nc1->start_pnt][y])/2;

 nc1->alpha = angle(new_point_x-centerpnt[x], new_point_y-centerpnt[y]);

 nc1->dist = 0; 
 nc1->dist += (hullpnts[pntcount][x] - hullpnts[nc1->start_pnt][x])*
	      (hullpnts[pntcount][x] - hullpnts[nc1->start_pnt][x]);
 nc1->dist += (hullpnts[pntcount][y] - hullpnts[nc1->start_pnt][y])*
	      (hullpnts[pntcount][y] - hullpnts[nc1->start_pnt][y]);

 nc2->start_pnt = pntcount;
 nc2->end_pnt   = largest_crv->end_pnt;

 new_point_x = (hullpnts[nc2->end_pnt][x] + hullpnts[nc2->start_pnt][x])/2;
 new_point_y = (hullpnts[nc2->end_pnt][y] + hullpnts[nc2->start_pnt][y])/2;

 nc2->alpha  = angle(new_point_x-centerpnt[x], new_point_y-centerpnt[y]);

 nc2->dist = 0; 
 nc2->dist += (hullpnts[nc2->end_pnt][x] - hullpnts[pntcount][x])*
	      (hullpnts[nc2->end_pnt][x] - hullpnts[pntcount][x]);
 nc2->dist += (hullpnts[nc2->end_pnt][y] - hullpnts[pntcount][y])*
	      (hullpnts[nc2->end_pnt][y] - hullpnts[pntcount][y]);

 last_alpha = largest_crv->alpha;

 tmp = largest_crv;
 largest_crv = largest_crv->next;
 free(tmp);

 if(fabs(nc1->alpha-last_alpha) > minimum_alpha)
   add_to_crv_list(nc1);
 else
   free(nc1);
 if(fabs(nc2->alpha-last_alpha) > minimum_alpha)
   add_to_crv_list(nc2);
 else
   free(nc2);
 return 1;
}


static show_list()
{
 REGION_CRV *tmp=largest_crv;

 printf("\n");
 while(tmp!=NULL) {
  printf("start=%d\tend=%d\tdist= %5.5lg\talpha=%5.2lf\n", 
          tmp->start_pnt, tmp->end_pnt, tmp->dist, 180*tmp->alpha/M_PI);
  tmp=tmp->next;
 }
 printf("\n");
}


#if DEBUG
static int addpoint_debug(double *pc, double*po)
{
 int i;

 pntcount++;
 for (i=1; i<=dim; i++)
   hullpnts[pntcount][i]=po[i];

 return 1;
}
#endif


static int pointcomp(REGION_PNT *rp1, REGION_PNT *rp2)
{
 if(rp1->alpha > rp2->alpha)
  return 1;
 else
 if(rp1->alpha < rp2->alpha)
  return -1;
 else
  return 0;
}


static void set_and_print_point(double alpha, int x, int y, 
				REGION_PNT *region_points)
{
 	region_points[pntcount].x=hullpnts[pntcount][x]*scale[x];
	region_points[pntcount].y=hullpnts[pntcount][y]*scale[y];
	region_points[pntcount].alpha=180*alpha/M_PI;

	printf("%5.2lf\t%9.5lf\t%9.5lf\t",
		180*alpha/M_PI,
		hullpnts[pntcount][x]*scale[x],
		hullpnts[pntcount][y]*scale[y]);

	region_points[pntcount].mx=
		100.0*(hullpnts[pntcount][x]-centerpnt[x])/centerpnt[x];
	region_points[pntcount].my=
		100.0*(hullpnts[pntcount][y]-centerpnt[y])/centerpnt[y];
	printf("%5.2lf\t%5.2lf\n", 
		region_points[pntcount].mx,
		region_points[pntcount].my);

}


int get_regions()
{
 int x, y;
 int i, k;
 double xbound_x, xbound_y, ybound_x, ybound_y;
 double alpha, delta_alpha=M_PI/4;
 int points_nr;
 REGION_CRV *tmp;
 PARAMETER_SET *tmp_param_set=parameter_list;

 FILE *fp;
 char outfile[LINE_LENGTH];
 char errmsg[LINE_LENGTH];
 char outline[LONG_LINE_LENGTH];
 
 double marginsx, marginsy;
 double *po, *pc;
 REGION_PNT *region_points;

 double minx_x, minx_y, maxx_x, maxx_y;
 int min_i, max_i;

 double new_point_x, new_point_y;


 	while (tmp_param_set!=NULL) {

	x=tmp_param_set->x;
	y=tmp_param_set->y;
	points_nr=tmp_param_set->points_nr;

	strcpy(outfile, param.output_dir);
	strcat(outfile, param.circuit_name);
	strcat(outfile, param.region_ext);
	strcat(outfile, param.opt_num);
	i=strlen(outfile);
	sprintf(&outfile[i], ".%d_%d", x, y);

        if ((fp=fopen(outfile,"w")) == NULL) {
                strcpy(errmsg, "Can't write to the '");
		strcat(errmsg, outfile);
		strcat(errmsg, "' file.\n");
                nrerror(errmsg);
        }

	pc=vector(1,dim);
	po=vector(0,dim);
        hullpnts  = matrix(1,points_nr+1,0,dim);
	region_points = (REGION_PNT *)calloc(points_nr+1, sizeof(REGION_PNT));
	pntcount=0;


#if DEBUG
        printf("%9.5f\t%9.5f\t%9.5f\t%9.5f\n\n\n",
                         lower[x], upper[x], lower[y], upper[y]);
#endif

        minimum_alpha = 2*M_PI/MINIMUM_ALPHA_DIVIDER;

/* alpha = 0 */
        alpha = 0;
        for (i=1; i<=dim; i++)
           pc[i]=po[i]=centerpnt[i];

	po[x] = upper[x];
	po[y] = centerpnt[y];

#if DEBUG_ADD
        if (!addpoint_debug(pc,po))
         	nrerror("parameter values failed");
#else
        if (!addpoint(pc,po))
         	nrerror("parameter values failed");
#endif
       	 
	set_and_print_point(alpha, x, y, region_points);


	for(alpha=M_PI/2; alpha<=2*M_PI; alpha+=M_PI/2) {

         	tmp = (REGION_CRV*)calloc(1, sizeof(REGION_CRV));
         	tmp->start_pnt = pntcount;
 	 	tmp->end_pnt = 1+pntcount%4;


#if DEBUG_PREVIOUS
 	 	tmp->alpha = alpha - delta_alpha;
         	tmp->delta = delta_alpha/2;
#endif

		if(tmp->end_pnt!=1) {

        		for (i=1; i<=dim; i++)
           			pc[i]=po[i]=centerpnt[i];

			if(alpha == M_PI/2) {
				po[x] = centerpnt[x];
				po[y] = upper[y];
			}
			else if(alpha == M_PI) {
				po[x] = lower[x];
				po[y] = centerpnt[y];
			} else if (alpha==3*M_PI/2) {
				po[x] = centerpnt[x];
				po[y] = lower[y];
			}


#if DEBUG_ADD
        		if (!addpoint_debug(pc,po))
         			nrerror("parameter values failed");
#else
        		if (!addpoint(pc,po))
         			nrerror("parameter values failed");
#endif
			set_and_print_point(alpha, x, y, region_points);
		}

 new_point_x = (hullpnts[tmp->end_pnt][x] + hullpnts[tmp->start_pnt][x])/2;
 new_point_y = (hullpnts[tmp->end_pnt][y] + hullpnts[tmp->start_pnt][y])/2;

 tmp->alpha = angle(new_point_x-centerpnt[x], new_point_y-centerpnt[y]);


         	tmp->dist = 0; 
         	tmp->dist += (hullpnts[tmp->end_pnt][x] 
			     	     - hullpnts[tmp->start_pnt][x])*
	      		     	     (hullpnts[tmp->end_pnt][x] 
			      	     - hullpnts[tmp->start_pnt][x]);
         	tmp->dist += (hullpnts[tmp->end_pnt][y]
 			      	     - hullpnts[tmp->start_pnt][y])*
	     		     	     (hullpnts[tmp->end_pnt][y] 
			      	     - hullpnts[tmp->start_pnt][y]);
         	add_to_crv_list(tmp);
#if DEBUG
		show_list();
#endif
        } /* for */


	for(k=0; k<points_nr-4; k++) {

        	for (i=1; i<=dim; i++)
           		pc[i]=po[i]=centerpnt[i];


		alpha = largest_crv->alpha;

		if((alpha<M_PI/2) || (alpha>3*M_PI/2))
                	xbound_x = upper[x];
		else
			xbound_x = lower[x];
		if(alpha<M_PI)
                	ybound_y = upper[y];
		else
			ybound_y = lower[y];

                xbound_y=centerpnt[y]+tan(alpha)*(xbound_x-centerpnt[x]);

		if(fabs(xbound_y-centerpnt[y]) <=fabs(ybound_y-centerpnt[y])) {
				po[x] = xbound_x;
				po[y] = xbound_y;
		}
		else {
			ybound_x = 
				 (ybound_y - centerpnt[y])/tan(alpha) + 
			  	 centerpnt[x];
			po[x] = ybound_x;
			po[y] = ybound_y;
		}

#if DEBUG_ADD
        	if (!addpoint_debug(pc,po))
         		nrerror("parameter values failed");
#else
        	if (!addpoint(pc,po))
         		nrerror("parameter values failed");
#endif

		set_and_print_point(alpha, x, y, region_points);
		modify_region_crv_list(x,y);
#if DEBUG
		show_list();
#endif
	}  /* for */

 	qsort(&region_points[1], points_nr, sizeof(REGION_PNT), 
		(int (*)())pointcomp);


	for(i=0; i<=points_nr; i++) {
         k=1+i%points_nr;
	 fprintf(fp, "%5.2lf\t%9.5lf\t%9.5lf\t%5.2lf\t%5.2lf\n", 
		region_points[k].alpha, 
		region_points[k].x,
		region_points[k].y,
		region_points[k].mx,
		region_points[k].my);
	}

	free_vector(pc, 1,dim);
	free_vector(po, 0,dim);
        free_matrix(hullpnts,1,points_nr+1,0,dim);
	free(region_points);
	fclose(fp);

	printf("\n\n");

        tmp_param_set=tmp_param_set->next;
   } /* while */
}

