#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "constype.h"

#include "timelogic.h"

int print_signals(SIGNALS *sp, char * filename)
{
 int k;
 PULSE *pp;
 FILE *fp;

 fp=fopen(filename, "w");
 if(fp==NULL)
   return 0;

 for(k=0; k<sp->signal_num; k++)
  {
   fprintf(fp, "%s\n\n", sp->signal_names[k]);

   pp=sp->sig_bgn[k];

   while(pp!=NULL)
    {
     fprintf(fp, "%.2f ps\t%.2f uV\t%.2f\n", pp->t*1.0e12, 
                                             pp->val*1.0e+6, 
                                             pp->phase);
     pp=pp->next;
    }

   fprintf(fp, "\n\n");
  }

 fclose(fp);

 return 1;
}


int print_edges(SIGNALS *sp, char * filename)
{
 int k;
 EDGE *pp;
 FILE *fp;

 fp=fopen(filename, "w");
 if(fp==NULL)
   return 0;

 for(k=0; k<sp->signal_num; k++)
  {
   fprintf(fp, "%s\n\n", sp->signal_names[k]);

   pp=sp->sig_edge_bgn[k];

   fprintf(fp, "edge\ttime\n");
   while(pp!=NULL)
    {
     fprintf(fp, "%s\t%.2f ps\n", 
					     (pp->direction?("F"):("R")),
					     pp->exact_time*1.0e12
                                             );
     pp=pp->next;
    }

   fprintf(fp, "\n\n");
  }

 fclose(fp);

 return 1;
}



void print_delays(FILE *fp, SIGNALS *sp, INPUTS *ip)
{
 INPUTS *np=ip;
 DELAYS *dp;
 int i,k;

 fprintf(fp, "DELAYS:\n\n");
 for(i=sp->clk_num; i<sp->clk_num+sp->ins_num; i++)
     fprintf(fp, "%s\t", sp->signal_names[i]);
 fprintf(fp, "delay (input clk_cycle)\n\n");

 while(np!=NULL)
  {
   for(i=0; i<sp->ins_num; i++)
     fprintf(fp, "%d\t", np->input_val[i]);

   dp=np->delay;
   i=1;
   while(dp!=NULL)
    {
     if(i==4)
       {
        fprintf(fp, "\n");
        for(k=0; k<sp->ins_num; k++)
         fprintf(fp, "\t");
        i=1;
       }
     fprintf(fp, "%.3f ps\t(%d)\t", dp->delta*1.0e12, dp->clk_cycle);
     i++;
     dp=dp->next;
    }
   fprintf(fp, "\n\n");
   np=np->next;
  }
}


static void compute_min_max_voltage(SIGNALS *sp, MIN_MAX_AVR *result)
{
 int k, pulse_nr=0;
 PULSE *pp;

 result->v.min=10.0; /* 10 V */
 result->v.max=0;
 result->v.avr=0;

  for(k=0; k<sp->signal_num; k++)
  {
   pp=sp->sig_bgn[k];

   while(pp!=NULL)
    {
     result->v.avr += pp->val;
     pulse_nr++;
     if(pp->val<result->v.min)
       result->v.min = pp->val;
     if(pp->val>result->v.max)
       result->v.max = pp->val;
     pp=pp->next;
    }
  }
 if(pulse_nr!=0)
  result->v.avr/=pulse_nr;
 else
  result->v.min = 0;
}


void print_voltages(FILE *fp, MIN_MAX_AVR *peak_volt)
{
 fprintf(fp, "PEAK VOLTAGES:\n\n");
 fprintf(fp, "min - %.2f uV\n", peak_volt->v.min*1.0e6);
 fprintf(fp, "max - %.2f uV\n", peak_volt->v.max*1.0e6);
 fprintf(fp, "avr - %.2f uV\n", peak_volt->v.avr*1.0e6);
}


int print_with_clk(SIGNALS *sp, FILE *fp)
{
 int clk_cycle, sig_nr;
 int tval = 0;
 int next_line;
 INPUTS *ip;
 MIN_MAX_AVR peak_volt;
 
 fprintf(fp, "CIRCUIT NAME:\t%s\n\n", param.circuit_name);
 fprintf(fp, "simulation step   = %.2f ps\nsimulation length = %.2f ps\n\n",  
         sp->min_time_delta*1.0e12,
         (sp->indices_nr-1)*(sp->min_time_delta*1.0e12));

 fprintf(fp, "SIGNALS TRACED:\n");
 if(param.pulse_extraction_method==COLLECT_BY_VOLTAGE)
  {
   for(sig_nr=0; sig_nr<sp->signal_num; sig_nr++)
     fprintf(fp, "%s\t<=>\t%s,\t%s\n", 
                 sp->signal_names[sig_nr],
                 sp->jspice_voltage_names[sig_nr],
                 sp->jspice_phase_names[sig_nr]);
  }
 else
  {
   for(sig_nr=0; sig_nr<sp->signal_num; sig_nr++)
     fprintf(fp, "%s\t<=>\t%s\n", 
                 sp->signal_names[sig_nr],
                 sp->jspice_phase_names[sig_nr]);
  }
 fprintf(fp, "\n\n");


 fprintf(fp, "CYCLE\tCLK\tPERIOD\t");

 for(sig_nr=1; sig_nr<sp->signal_num; sig_nr++)
   fprintf(fp, "\t%s", sp->signal_names[sig_nr]);
 fprintf(fp, "\n");

 for(clk_cycle=0; clk_cycle<sp->clk_cycles_nr; clk_cycle++)
  {
   if(clk_cycle!=0)
     fprintf(fp, "\t%.2f\t\t", sp->clk_times[clk_cycle]*1.0e12);
   else
     fprintf(fp, "\t\t\t");

   for(sig_nr=1; sig_nr<sp->signal_num; sig_nr++)
    if(sp->sig_log[sig_nr][clk_cycle].val>0)
      fprintf(fp, "\t +%.2f", sp->sig_log[sig_nr][clk_cycle].delta[0]*1.0e12);
    else
      fprintf(fp, "\t");
   fprintf(fp, "\n");

   tval = 1;
   while(1) {
     next_line = 0;
     for(sig_nr=1; sig_nr<sp->signal_num; sig_nr++)
       if(sp->sig_log[sig_nr][clk_cycle].val>tval)
         next_line = 1;

     if(!next_line)
       break;

     fprintf(fp, "\t\t\t");
     for(sig_nr=1; sig_nr<sp->signal_num; sig_nr++)
       if(sp->sig_log[sig_nr][clk_cycle].val>tval)
         fprintf(fp, "\t +%.2f",
                      sp->sig_log[sig_nr][clk_cycle].delta[tval]*1.0e12);
       else
         fprintf(fp, "\t");
     fprintf(fp, "\n");

     tval++;
   }

   if((clk_cycle!=0) && (clk_cycle != sp->clk_cycles_nr-1))
     fprintf(fp,"%d\t\t%.2f\t", clk_cycle, 
         (sp->clk_times[clk_cycle+1] - sp->clk_times[clk_cycle])*1.0e12);
   else
     fprintf(fp,"%d\t\t\t", clk_cycle);
   for(sig_nr=1; sig_nr<sp->signal_num; sig_nr++)
     fprintf(fp, "\t%d", sp->sig_log[sig_nr][clk_cycle].val);
   fprintf(fp, "\n");
  }

 fprintf(fp, "\n\n");
 if((ip=compute_delays(sp))!=NULL)
   print_delays(fp, sp, ip);  

 if(param.pulse_extraction_method==COLLECT_BY_VOLTAGE)
  {
   compute_min_max_voltage(sp, &peak_volt);
   print_voltages(fp, &peak_volt);
  }

 return 1;
}



int print_async_signals(SIGNALS *sp, FILE *fp)
{
 int sig_nr;
 MIN_MAX_AVR peak_volt;
 int current_index, min_step, min_sig_nr;
 PULSE **last_pulse;

 int k;
 PULSE *pp;


 fprintf(fp, "simulation step   = %.2f ps\nsimulation length = %.2f ps\n\n",  
         sp->min_time_delta*1.0e12,
         (sp->indices_nr-1)*(sp->min_time_delta*1.0e12));


 fprintf(fp, "SIGNALS TRACED:\n");
 if(param.pulse_extraction_method==COLLECT_BY_VOLTAGE)
  {
   for(sig_nr=0; sig_nr<sp->signal_num; sig_nr++)
     fprintf(fp, "%s\t<=>\t%s,\t%s\n", 
                 sp->signal_names[sig_nr],
                 sp->jspice_voltage_names[sig_nr],
                 sp->jspice_phase_names[sig_nr]);
  }
 else
  {
   for(sig_nr=0; sig_nr<sp->signal_num; sig_nr++)
     fprintf(fp, "%s\t<=>\t%s\n", 
                 sp->signal_names[sig_nr],
                 sp->jspice_phase_names[sig_nr]);
  }
 fprintf(fp, "\n\n");


 fprintf(fp, "TIME\t\t");

 for(sig_nr=0; sig_nr<sp->ins_num+sp->inter_num; sig_nr++)
   fprintf(fp, "\t%s", sp->signal_names[sig_nr]);
 fprintf(fp, "\n\n");

 last_pulse = (PULSE**)calloc(sp->signal_num, sizeof(PULSE*));
 for(sig_nr=0; sig_nr<sp->signal_num; sig_nr++)
   last_pulse[sig_nr] = sp->sig_bgn[sig_nr];

 current_index = 0;
 do
  {
   min_step = sp->indices_nr+1;
   for(sig_nr=0; sig_nr<sp->signal_num; sig_nr++)
    {
     if(last_pulse[sig_nr]==NULL)
      continue;

     if(last_pulse[sig_nr]->index-current_index < min_step)
      {
       min_step = last_pulse[sig_nr]->index-current_index;
       min_sig_nr = sig_nr;
      }
    }

   if(min_step == sp->indices_nr+1)
     break;

   current_index = last_pulse[min_sig_nr]->index;

   if(min_sig_nr<sp->ins_num+sp->inter_num)
    {
     fprintf(fp, "%.2f\t\t", last_pulse[min_sig_nr]->t*1.0e12);
     for(sig_nr=0; sig_nr<sp->ins_num+sp->inter_num; sig_nr++)
      {
       if(last_pulse[sig_nr]==NULL)
         fprintf(fp, "\t0");
       else
       if(last_pulse[sig_nr]->index==current_index)
        {
         fprintf(fp, "\t1");
         last_pulse[sig_nr]=last_pulse[sig_nr]->next;
        }
       else
         fprintf(fp, "\t0");
      }
    }
   else
    {
     fprintf(fp, "%.2f\t\t%s", last_pulse[min_sig_nr]->t*1.0e12,
                               sp->signal_names[min_sig_nr]);
     for(sig_nr=0; sig_nr<sp->ins_num+sp->inter_num; sig_nr++)
      {
       if(last_pulse[sig_nr]!=NULL)
        {
         if(last_pulse[sig_nr]->prev!=NULL)
           fprintf(fp, "\t %.2f", 
            (last_pulse[min_sig_nr]->t - last_pulse[sig_nr]->prev->t)*1.0e12);
         else
           fprintf(fp, "\t-");
        }
       else
        {
         if(sp->sig_end[sig_nr]!=NULL)
           fprintf(fp, "\t %.2f", 
            (last_pulse[min_sig_nr]->t - sp->sig_end[sig_nr]->t)*1.0e12);
         else
           fprintf(fp, "\t-");
        }
      }
     last_pulse[min_sig_nr] = last_pulse[min_sig_nr]->next;
    }
   fprintf(fp, "\n\n");
  }
 while(1);


 if(param.pulse_extraction_method==COLLECT_BY_VOLTAGE)
  {
   compute_min_max_voltage(sp, &peak_volt);
   print_voltages(fp, &peak_volt);
  }

 return 1;
}




int print_holdsetup(char *filename, SIGNALS* sp_good, 
                    SIGNALS* sp_bad, HOLDSETUP *holdset, int hold1setup0)
{
 FILE *fp;
 int k;

 fp=fopen(filename, "a");
 if(fp==NULL)
   return 0;

 if(hold1setup0)
  {
   printf("HOLD TIME");
   fprintf(fp, "HOLD TIME");
  }
 else
  {
   printf("SETUP TIME");
   fprintf(fp, "SETUP TIME");
  }

 printf("= %.2f ps\n", holdset->delta*1.0e12);
 fprintf(fp, "= %.2f ps\n", holdset->delta*1.0e12);

 printf("on input:  %s\n", sp_good->signal_names[holdset->input]);
 fprintf(fp, "on input:  %s\n", sp_good->signal_names[holdset->input]);
 printf("clk cycle: %d\n", holdset->cycle);
 fprintf(fp, "clk cycle: %d\n", holdset->cycle);


 printf("for:\n");
 fprintf(fp,"for:\n");

 for(k=sp_good->clk_num; k<sp_good->clk_num+sp_good->ins_num; k++)
  {
   printf("\t%s = %d\n", sp_good->signal_names[k],
                        sp_good->sig_log[k][holdset->cycle].val);
   fprintf(fp,"\t%s = %d\n", sp_good->signal_names[k],
                        sp_good->sig_log[k][holdset->cycle].val);
  }

 printf("proper output:\n\t%s = %d\n",
         sp_good->signal_names[sp_good->signal_num-1],
         sp_good->sig_log[sp_good->signal_num-1][holdset->cycle+1].val);
 fprintf(fp,"proper output:\n\t%s = %d\n",
         sp_good->signal_names[sp_good->signal_num-1],
         sp_good->sig_log[sp_good->signal_num-1][holdset->cycle+1].val);

 printf("false  output:\n\t%s = %d\n\n",
         sp_bad->signal_names[sp_bad->signal_num-1],
         sp_bad->sig_log[sp_bad->signal_num-1][holdset->cycle+1].val);
 fprintf(fp,"false  output:\n\t%s = %d\n\n",
         sp_bad->signal_names[sp_bad->signal_num-1],
         sp_bad->sig_log[sp_bad->signal_num-1][holdset->cycle+1].val);

 if(sp_good->sig_log[sp_good->signal_num-1][holdset->cycle+1].val>0)
  {
   if(hold1setup0)
    {
     printf("HOLD PROPAGATION DELAY = ");
     fprintf(fp,"HOLD PROPAGATION DELAY = ");
    }
   else
    {
     printf("SETUP PROPAGATION DELAY = ");
     fprintf(fp,"SETUP PROPAGATION DELAY = ");
    }
   printf("%.1f ps\n",
    sp_good->sig_log[sp_good->signal_num-1][holdset->cycle+1].delta[0]*1.0e12);
   fprintf(fp, "%.1f ps\n",
    sp_good->sig_log[sp_good->signal_num-1][holdset->cycle+1].delta[0]*1.0e12);
  }
  

 fclose(fp);
 return 1;
}


int print_xy_propagation(FILE *fp, SIGNALS *sp, int input_nr, 
                         int in_clk_cycle, int out_clk_cycle)
{ 
 fprintf(fp, "%.4f\t",
         sp->sig_log[sp->clk_num+input_nr-1][in_clk_cycle].delta[0]*1.0e12);
 fprintf(fp, "%.4f\n",
         sp->sig_log[sp->signal_num-1][out_clk_cycle].delta[0]*1.0e12);
}


static int print_interpolated_time(SIGNALS *sp_phase, char * filename)
{
 int k;
 PULSE *pp_phase;
 FILE *fp;

 fp=fopen(filename, "w");
 if(fp==NULL)
   return 0;

 for(k=0; k<sp_phase->signal_num; k++)
  {
   fprintf(fp, "%s\n\n", sp_phase->signal_names[k]);

   pp_phase=sp_phase->sig_bgn[k];

   while(pp_phase!=NULL)
    {
     fprintf(fp, "%.5f\n", pp_phase->t*1.0e12);

     pp_phase=pp_phase->next;
    }
   fprintf(fp, "\n\n");
  }
 fclose(fp);

 return 1;
}


static int print_interpolated_signals(SIGNALS *sp_phase, char * filename)
{
 int k;
 PULSE *pp_phase;
 FILE *fp;
 char outfile[NAME_LENGTH];

 for(k=0; k<sp_phase->signal_num; k++)
  {
   strcpy(outfile, filename);
   strcat(outfile, ".");
   strcat(outfile, sp_phase->signal_names[k]);

   fp=fopen(outfile, "w");
   if(fp==NULL)
    {
     printf("Cannot open file: %s\n", outfile);
     continue;
    }

   pp_phase=sp_phase->sig_bgn[k];
   while(pp_phase!=NULL)
    {
     if(pp_phase->next!=NULL)
       fprintf(fp, "%.5f\n", (pp_phase->next->t-pp_phase->t)*1.0e12);
     pp_phase=pp_phase->next;
    }
   fclose(fp);
  }

 return 1;
}


static int print_interpolated_time_phase(SIGNALS *sp_phase, char * filename)
{
 int k;
 PULSE *pp_phase;
 FILE *fp;

 fp=fopen(filename, "w");
 if(fp==NULL)
   return 0;


 for(k=0; k<sp_phase->signal_num; k++)
  {
   fprintf(fp, "%s\n\n", sp_phase->signal_names[k]);

   pp_phase=sp_phase->sig_bgn[k];

   fprintf(fp, "PHASE BELOW\t");
   fprintf(fp, "PHASE ABOVE\t");
   fprintf(fp, "PHASE THRESH\t");
   fprintf(fp, "TIME BELOW\t");
   fprintf(fp, "TIME ABOVE\t");
   fprintf(fp, "TIME THRESH\n");

   while(pp_phase!=NULL)
    {
     fprintf(fp, "%.2f\t\t", pp_phase->p_below);
     fprintf(fp, "%.2f\t\t", pp_phase->p_above);
     fprintf(fp, "%.2f\t\t", pp_phase->norm_phase+param.nom_phase_thresh);
     fprintf(fp, "%.2f\t\t", pp_phase->t_below*1.0e12);
     fprintf(fp, "%.2f\t\t", pp_phase->t_above*1.0e12);
     fprintf(fp, "%.2f\n", pp_phase->t*1.0e12);

     pp_phase=pp_phase->next;
    }
   fprintf(fp, "\n\n");
  }
 fclose(fp);

 return 1;
}



#define PHASE_TYPE 0
#define TIME_TYPE  1
#define VOLTAGE_TYPE 2


static void set_min_max_avr(MIN_MAX_AVR * mma, double new_val, int type)
{
 switch(type)
   {
    case TIME_TYPE:
         if(new_val< mma->t.min)
           mma->t.min = new_val;
         if(new_val > mma->t.max)
           mma->t.max = new_val;
         mma->t.avr += new_val;
         break;

    case PHASE_TYPE:
         if(new_val< mma->p.min)
           mma->p.min = new_val;
         if(new_val > mma->p.max)
           mma->p.max = new_val;
         mma->p.avr += new_val;
         break;

    case VOLTAGE_TYPE:
         if(new_val< mma->v.min)
           mma->v.min = new_val;
         if(new_val > mma->v.max)
           mma->v.max = new_val;
         mma->v.avr += new_val;
         break;
   }
}


int print_phase_peak_signals(SIGNALS *sp_phase, SIGNALS* sp_peak, char * filename)
{
 int k, i, pulse_nr;
 PULSE *pp_phase, *pp_peak;
 FILE *fp;
 int OK=1;

 TIME tdelta;
 PHASE pdelta;
 VOLTAGE vdelta;
 MIN_MAX_AVR t2peak, phaseper, peakper, v2peak, p2phase, p2thresh;


 if(sp_peak->signal_num != sp_phase->signal_num)
  {
   printf("Error !\n");
   printf("Numbers of signals defined in voltage and phase files do not match!\n");
   return 0;
  }

 if(sp_peak->min_time_delta != sp_phase->min_time_delta)
  {
   printf("Error !\n");
   printf("Time resolutions defined in voltage and phase files do not match!\n");
   return 0;
  }

 fp=fopen(filename, "w");
 if(fp==NULL)
   return 0;


 for(k=0; k<sp_peak->signal_num; k++)
  {
   fprintf(fp, "%s\n\n", sp_peak->signal_names[k]);

   t2peak.t.min = phaseper.t.min = peakper.t.min = 1.0;
   t2peak.t.max = phaseper.t.max = peakper.t.max = -1.0;
   t2peak.t.avr = phaseper.t.avr = peakper.t.avr = 0.0;
   p2phase.p.min = p2thresh.p.min = 2*M_PI;
   p2phase.p.max = p2thresh.p.max = -2*M_PI;
   p2phase.p.avr = p2thresh.p.avr = 0.0;
   v2peak.v.min = 1.0;
   v2peak.v.max = -1.0;
   v2peak.v.avr = 0;


   pp_phase=sp_phase->sig_bgn[k];
   pp_peak=sp_peak->sig_bgn[k];


   fprintf(fp, "PHASE TIME\t\t\t\t\t\t\t");
   fprintf(fp, "PHASE VOLTAGE\t\t");
   fprintf(fp, "PEAK PHASE\t\t");
   fprintf(fp, "PHASE\n");

   fprintf(fp, "\tTO PEAK\t\t");
   fprintf(fp, "PHASE PERIOD\t");
   fprintf(fp, "PEAK PERIOD\t\t\t");
   fprintf(fp, "TO PEAK\t\t\t");
   fprintf(fp, "TO PHASE\t\t");
   fprintf(fp, "TO THRESHOLD\n\n");

   pulse_nr=0;
   while(pp_phase!=NULL)
    {
     if(pp_peak==NULL)
      {
       printf("PULSE COLLECTION METHODS COMPARISON:\n");
       printf("Warning !\n");
       printf("Numbers of pulses in signal %s do not match.\n\n", 
                        sp_peak->signal_names[k]);
       OK=0;
       break;
      }

     fprintf(fp, "%.2f ps \t\t\t\t\t\t\t", pp_phase->t*1.0e12);
     fprintf(fp, "%.1f uV  \t\t", pp_phase->val*1.0e+6);
     fprintf(fp, "%.2f\t\t\t", pp_peak->phase);
     fprintf(fp, "%.2f", pp_phase->phase);
     fprintf(fp, "\n");

     tdelta = pp_phase->t-pp_peak->t;
     fprintf(fp, "\t%.2f ps \t", tdelta*1.0e12);
     set_min_max_avr(&t2peak, tdelta, TIME_TYPE);

     if(pp_phase->next!=NULL)
      {
       tdelta = pp_phase->next->t-pp_phase->t;
       fprintf(fp, "%.2f ps \t", tdelta*1.0e12);
       set_min_max_avr(&phaseper, tdelta, TIME_TYPE);
      }
     else
       fprintf(fp, "\t\t\t\t");

     if(pp_peak->next!=NULL)
      {
       tdelta = pp_peak->next->t-pp_peak->t;
       fprintf(fp, "%.2f ps \t\t\t", tdelta*1.0e12);
       set_min_max_avr(&peakper, tdelta, TIME_TYPE);
      }
     else
       fprintf(fp, "\t\t");

     vdelta = pp_phase->val-pp_peak->val;
     fprintf(fp, "%.1f uV  \t\t", vdelta*1.0e+6);
     set_min_max_avr(&v2peak, vdelta, VOLTAGE_TYPE);

     pdelta = pp_peak->phase-pp_phase->phase;
     fprintf(fp, "%.2f\t\t\t", pdelta);
     set_min_max_avr(&p2phase, pdelta, PHASE_TYPE);

     pdelta = pp_phase->phase-(pp_phase->norm_phase+param.nom_phase_thresh);
     fprintf(fp, "%.2f", pdelta);
     set_min_max_avr(&p2thresh, pdelta, PHASE_TYPE);
     fprintf(fp, "\n");

     pulse_nr++;
     pp_phase=pp_phase->next;
     pp_peak=pp_peak->next;
    }

   for(i=0; i<131; i++)
     fprintf(fp, "-");
   fprintf(fp, "\n");

   if(pulse_nr!=0)
    {
     t2peak.t.avr/=pulse_nr;
     p2phase.p.avr/=pulse_nr;
     p2thresh.p.avr/=pulse_nr;
     v2peak.v.avr/=pulse_nr;
    }
   if(pulse_nr>1)
    {
     phaseper.t.avr/=(pulse_nr-1);
     peakper.t.avr/=(pulse_nr-1);
    }

   fprintf(fp, "AVR\t");
   if(pulse_nr!=0)
     fprintf(fp, "%.2f ps \t", t2peak.t.avr*1.0e+12);
   else
     fprintf(fp, "\t\t");
   if(pulse_nr>1)
     fprintf(fp, "%.2f ps \t%.2f ps \t\t\t",
                 phaseper.t.avr*1.0e+12,
                 peakper.t.avr*1.0e+12);
   if(pulse_nr!=0)
    {
     fprintf(fp, "%.1f uV  \t\t", v2peak.v.avr*1.0e6);
     fprintf(fp, "%.2f\t\t\t", p2phase.p.avr);
     fprintf(fp, "%.2f", p2thresh.p.avr);
    }
   fprintf(fp, "\n");

   fprintf(fp, "MIN\t");
   if(pulse_nr!=0)
     fprintf(fp, "%.2f ps \t", t2peak.t.min*1.0e+12);
   else
     fprintf(fp, "\t\t");
   if(pulse_nr>1)
     fprintf(fp, "%.2f ps \t%.2f ps \t\t\t",
                 phaseper.t.min*1.0e+12,
                 peakper.t.min*1.0e+12);
   if(pulse_nr!=0)
    {
     fprintf(fp, "%.1f uV  \t\t", v2peak.v.min*1.0e6);
     fprintf(fp, "%.2f\t\t\t", p2phase.p.min);
     fprintf(fp, "%.2f", p2thresh.p.min);
    }
   fprintf(fp, "\n");

   fprintf(fp, "MAX\t");
   if(pulse_nr!=0)
     fprintf(fp, "%.2f ps \t", t2peak.t.max*1.0e+12);
   else
     fprintf(fp, "\t\t");
   if(pulse_nr>1)
     fprintf(fp, "%.2f ps \t%.2f ps \t\t\t", 
                 phaseper.t.max*1.0e+12,
                 peakper.t.max*1.0e+12);
   if(pulse_nr!=0)
    {
     fprintf(fp, "%.1f uV  \t\t", v2peak.v.max*1.0e6);
     fprintf(fp, "%.2f\t\t\t", p2phase.p.max);
     fprintf(fp, "%.2f", p2thresh.p.max);
    }
   fprintf(fp, "\n");

   fprintf(fp, "\n\n");
  }

 fclose(fp);

 return OK;
}



int interpolate_signal(SIGNALS *sp_phase, char *filename)
{
 char outfile[NAME_LENGTH];

 interpolate_phase_thresh_time(sp_phase);

 strcpy(outfile, filename);
 strcat(outfile, param.period_ext);
 return print_interpolated_signals(sp_phase, outfile);
}

